@@ -2,7 +2,7 @@
 """
 Module TokenConverter
 Sub-Package CLASSES of Package PLIB
-Copyright (C) 2008-2011 by Peter A. Donis
+Copyright (C) 2008-2012 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -44,8 +44,8 @@ class TokenConverter(abstractsequence):
     
     converters = []
     
-    def __init__(self, tokens=[]):
-        for index, token in enumerate(tokens):
+    def __init__(self, tokens=None):
+        for index, token in enumerate(tokens or []):
             self[index] = token
     
     def __len__(self):
