@@ -2,7 +2,7 @@
 """
 Module PANELS -- Dialog Classes for GUI
 Sub-Package GUI of Package PLIB -- Python GUI Framework
-Copyright (C) 2008-2011 by Peter A. Donis
+Copyright (C) 2008-2012 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -31,6 +31,7 @@ class PMainPanel(gui.PAutoPanel):
     
     align=ALIGN_JUST
     layout = LAYOUT_NONE
+    maintitle = None
     margin = specs.framemargin
     spacing = specs.panelspacing
     style=PANEL_NONE
@@ -43,3 +44,7 @@ class PMainPanel(gui.PAutoPanel):
         style = self.style
         gui.PAutoPanel.__init__(self, parent, layout=layout, style=style,
             align=align, margin=margin, spacing=spacing)
+        
+        # Set parent's caption from our maintitle attribute, if defined
+        if self.maintitle:
+            parent.set_caption(self.maintitle)
