@@ -35,13 +35,13 @@ class AbstractDictMixin(AbstractMappingMixin):
         raise NotImplementedError
     
     def __setitem__(self, key, value):
-        if key in self:
+        if key in self._keylist():
             self._set_value(key, value)
         else:
             self._add_key(key, value)
     
     def __delitem__(self, key):
-        if key in self:
+        if key in self._keylist():
             self._del_key(key)
         else:
             raise KeyError(repr(key))
