@@ -26,7 +26,7 @@ class AbstractKeyedMixin(object):
     """
     
     def _keylist(self):
-        """Return list of valid keys for this mapping.
+        """Return container of valid keys for this mapping.
         """
         raise NotImplementedError
     
@@ -39,19 +39,15 @@ class AbstractKeyedMixin(object):
         raise NotImplementedError
     
     def __getitem__(self, key):
-        if key in self:
+        if key in self._keylist():
             return self._get_value(key)
         raise KeyError(repr(key))
     
     def __iter__(self):
         return iter(self._keylist())
     
-    # These are reimplemented because they will be faster this way
-    # (we assume that _keylist returns a list-like object with
-    # similar performance for lookups and length checks)
-    
-    def __contains__(self, key):
-        return key in self._keylist()
+    # We expect this to be faster than the default implementation
+    # because _keylist() should be O(1) for length checks
     
     def __len__(self):
         return len(self._keylist())
