@@ -29,7 +29,7 @@ class AbstractMappingMixin(AbstractKeyedMixin):
         raise NotImplementedError
     
     def __setitem__(self, key, value):
-        if key in self:
+        if key in self._keylist():
             self._set_value(key, value)
         else:
             raise KeyError(repr(key))
