@@ -27,7 +27,7 @@ class CachedProperty(object):
     def __init__(self, fget, name=None, doc=None):
         self.__fget = fget
         self.__name = name or fget.__name__
-        self.__doc__ = doc
+        self.__doc__ = doc or fget.__doc__
     
     def __get__(self, instance, cls):
         # Note that for unbound method calls we don't cache,
