@@ -67,6 +67,9 @@ function split_string -- splits a string into three pieces, at the
 
 functions strtobool, strtodate -- self-explanatory.
 
+function type_from_name -- returns a type object corresponding
+to ``name``.
+
 constant universal_newline -- string representing the "universal
     newline" character
 
@@ -284,6 +287,19 @@ def strtodate(s):
     return datetime.date(*map(int, s.split('-')))
 
 
+def type_from_name(name):
+    """Return type object corresponding to ``name``.
+    
+    Currently searches only the built-in types. No checking is done to
+    make sure the returned object is actually a type.
+    """
+    import __builtin__
+    try:
+        return getattr(__builtin__, name)
+    except AttributeError:
+        raise ValueError, "no type corresponding to %s" % name
+
+
 ### Convenience function to upgrade builtin namespace ###
 
 BUILTIN_NAMES = [
@@ -390,8 +406,9 @@ def upgrade_builtins():
 # our sub-modules appear in our namespace
 
 excludes = ['_builtins', '_extras',
-    '_defaultdict', '_namedtuple', '_iters',
-    'coll', 'decotools', 'func', 'imp', 'iters', 'mail', 'mathlib', 'options', 'timer']
+    '_defaultdict', '_namedtuple', '_typed_namedtuple', '_iters',
+    'cmdline', 'coll', 'decotools', 'func', 'imp', 'iters',
+    'mail', 'mathlib', 'options', 'timer']
 
 ModuleProxy(__name__).init_proxy(__name__, __path__, globals(), locals(),
     excludes=excludes)
