@@ -1,9 +1,8 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """
 Module _EXTRAS -- Enhancing the builtin namespace
 Sub-Package STDLIB of Package PLIB
-Copyright (C) 2008-2011 by Peter A. Donis
+Copyright (C) 2008-2012 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -12,7 +11,7 @@ See the LICENSE and README files for more information
 from __future__ import generators # to ensure we can use yield in version 2.2
 
 import sys
-from operator import mul
+from operator import mul as _mul
 
 
 def first(iterable, default=None):
@@ -67,7 +66,7 @@ def last(iterable, default=None):
     return result
 
 
-def prod(iterable):
+def prod(iterable, mul=_mul):
     """Return the product of all items in iterable.
     """
     return reduce(mul, iterable, 1)
