@@ -35,12 +35,14 @@ from collections import *
 try:
     defaultdict
 except NameError:
-    from _defaultdict import defaultdict
+    from plib.stdlib._defaultdict import defaultdict
 
 try:
     namedtuple
 except NameError:
-    from _namedtuple import namedtuple
+    from plib.stdlib._namedtuple import namedtuple
+
+from plib.stdlib._typed_namedtuple import typed_namedtuple
 
 
 class fifo(deque):
@@ -59,3 +61,14 @@ class stack(list):
     def __init__(self, *args, **kwargs):
         self.next = self.pop
         list.__init__(self, *args, **kwargs)
+
+
+def merge_dict(target, source):
+    """Merges source into target
+    
+    Only updates keys not already in target.
+    """
+    
+    merges = dict((key, value) for key, value in source.iteritems()
+        if key not in target)
+    target.update(merges)
