@@ -8,9 +8,24 @@ Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
 """
 
+from plib.stdlib.coll import merge_dict
 from plib.stdlib.func import partial, wraps
 
 
+def wraps_class(cls, assigned=('__name__', '__module__'), updated=()):
+    """Decorator for wrapping classes with other classes.
+    
+    This decorator works like ``wraps``, but changes the
+    defaults so that the class's docstring and ``__dict__``
+    are not changed (the docstring isn't writable once the
+    class is constructed, and the dictionary shouldn't be
+    mutated since attributes are inherited from the base
+    class anyway). See the ``cached_class`` decorator below
+    for an example of typical usage.
+    """
+    return wraps(cls, assigned, updated)
+
+
 def cached_property(func):
     """Decorator for cached property.
     
@@ -18,6 +33,8 @@ def cached_property(func):
     decorator, but caches the property value in the instance
     dictionary so that the underlying function is only called
     once. The property is read-only.
+    
+    See the ``CachedProperty`` module for more information.
     """
     
     from plib.stdlib import CachedProperty
@@ -32,43 +49,187 @@ def cached_function(func):
     so are always hashable, but if any arguments (keyword
     or positional) are non-hashable, that set of arguments
     is not cached.
+    
+    Typical usage:
+    
+        >>> @cached_function
+        ... def f1(a, b=None):
+        ...     print "f1 called with", repr(a), repr(b)
+        ...     return a, b
+        ...
+        >>> @cached_function
+        ... def f2(a, b=None):
+        ...     print "f2 called with", repr(a), repr(b)
+        ...     return b, a
+        ...
+        >>> f1(1)
+        f1 called with 1 None
+        (1, None)
+        >>> f1(1)
+        (1, None)
+        >>> f2(1)
+        f2 called with 1 None
+        (None, 1)
+        >>> f2(1)
+        (None, 1)
+    
+    Note that we are also testing to make sure the cache
+    dictionary is separate for each function, even though
+    the argument names and values are the same:
+    
+        >>> f1(1, 2)
+        f1 called with 1 2
+        (1, 2)
+        >>> f2(1, 2)
+        f2 called with 1 2
+        (2, 1)
+        >>> f2(2, 1)
+        f2 called with 2 1
+        (1, 2)
+        >>> f1(2, 1)
+        f1 called with 2 1
+        (2, 1)
+    
+    Note also that the same argument may be cached twice if it
+    is passed both positionally and as a keyword; the decorator
+    does not check argument names, only how they are passed:
+    
+        >>> f1(1, 2)  # already called above
+        (1, 2)
+        >>> f2(1, 2)  # already called above
+        (2, 1)
+        >>> f1(1, b=2)
+        f1 called with 1 2
+        (1, 2)
+        >>> f2(1, b=2)
+        f2 called with 1 2
+        (2, 1)
+        >>> f1(a=1, b=2)
+        f1 called with 1 2
+        (1, 2)
+        >>> f2(a=1, b=2)
+        f2 called with 1 2
+        (2, 1)
+        >>> f2(2, 1)
+        (1, 2)
+        >>> f1(2, 1)
+        (2, 1)
+        >>> f2(2, b=1)
+        f2 called with 2 1
+        (1, 2)
+        >>> f1(2, b=1)
+        f1 called with 2 1
+        (2, 1)
+        >>> f2(a=2, b=1)
+        f2 called with 2 1
+        (1, 2)
+        >>> f1(a=2, b=1)
+        f1 called with 2 1
+        (2, 1)
     """
     
     cache = {}
+    sentinel = object()  # so we can cache None results
     
     @wraps(func)
     def _decorated(*args, **kwds):
         key = args + tuple(kwds.iteritems())
         try:
-            result = cache.get(key, None)
+            result = cache.get(key, sentinel)
         except TypeError:
             # Can't cache this set of arguments
-            result = key = None
-        if result is None:
+            result = key = sentinel
+        if result is sentinel:
             result = func(*args, **kwds)
-            if key is not None:
+            if key is not sentinel:
                 cache[key] = result
         return result
     
     return _decorated
 
 
+def cached_method(method):
+    """Decorator to cache method results by instance.
+    
+    This decorator combines the functionality of ``cached_property``
+    and ``cached_function``, allowing method results to be cached
+    by arguments while ensuring that the caches for separate
+    instances of a class do not overlap. Technically, ``cached_function``
+    by itself can do this, but using it directly might possibly have
+    issues when a class instance is not hashable (since the instance
+    appears as the ``self`` argument and so is part of the cache key).
+    
+    Typical usage:
+    
+        >>> class Test(object):
+        ...     @cached_method
+        ...     def test(self, *args, **kwds):
+        ...         print "Test instance has args", args, "and keywords", kwds
+        ...
+        >>> t = Test()
+        >>> t.test()
+        Test instance has args () and keywords {}
+        >>> t.test()
+        >>> t1 = Test()
+        >>> t1.test()
+        Test instance has args () and keywords {}
+        >>> t1.test()
+        >>> t.test(0)
+        Test instance has args (0,) and keywords {}
+        >>> t.test(0)
+        >>> t.test(a=0)
+        Test instance has args () and keywords {'a': 0}
+        >>> t.test(a=0)
+        >>> t1.test(0)
+        Test instance has args (0,) and keywords {}
+        >>> t1.test(0)
+        >>> t1.test(a=0)
+        Test instance has args () and keywords {'a': 0}
+        >>> t1.test(a=0)
+        >>> t.test(0, a=0)
+        Test instance has args (0,) and keywords {'a': 0}
+        >>> t.test(0, a=0)
+        >>> t1.test(0, a=0)
+        Test instance has args (0,) and keywords {'a': 0}
+        >>> t1.test(0, a=0)
+    
+    Other properties are similar to the ``cached_function``
+    decorator, applied separately for each instance of the
+    class. Note, though, that the above illustrates that
+    even results of ``None`` are cached; the ``cached_function``
+    decorator keeps an internal unique "sentinel" object to
+    distinguish a set of arguments that gives a ``None``
+    result from arguments not in the cache.
+    
+    See the ``CachedMethod`` module for more information.
+    """
+    
+    from plib.stdlib import CachedMethod
+    return CachedMethod(method)
+
+
 def cached_class(klass):
     """Decorator to cache class instances by constructor arguments.
     
-    This is basically the cached_function decorator applied to
-    the __new__ method, with a couple of extra features to make
-    it work properly with classes.
+    This is basically the ``cached_function`` decorator applied to
+    the ``__new__`` method, with a couple of extra features to make
+    it work properly with classes. Note that we do *not* use the
+    ``cached_method`` decorator on ``__new__``, since (a) that method
+    is not treated like other methods by Python, and (b) that method
+    *returns* a class instance, so we can't cache its results by
+    class instance, as ``cached_method`` does.
     
     Typical usage:
         
         >>> @cached_class
         ... class Test(object):
         ...     def __init__(self, *args, **kwds):
+        ...         print "Initializing with", args, kwds
         ...         self.args = args
         ...         self.kwds = kwds
         ...
         >>> t1 = Test(0, a=1)
+        Initializing with (0,) {'a': 1}
         >>> t1.args
         (0,)
         >>> t1.kwds
@@ -89,6 +250,7 @@ def cached_class(klass):
     matches are still new sets of arguments:
     
         >>> t2 = Test(1, b=2)
+        Initializing with (1,) {'b': 2}
         >>> t2.args
         (1,)
         >>> t2.kwds
@@ -96,6 +258,7 @@ def cached_class(klass):
         >>> t2 is t1
         False
         >>> t3 = Test(0, b=2)
+        Initializing with (0,) {'b': 2}
         >>> t3.args
         (0,)
         >>> t3.kwds
@@ -103,6 +266,7 @@ def cached_class(klass):
         >>> any(t3 is t for t in (t1, t2))
         False
         >>> t4 = Test(1, a=1)
+        Initializing with (1,) {'a': 1}
         >>> t4.args
         (1,)
         >>> t4.kwds
@@ -114,6 +278,7 @@ def cached_class(klass):
     set of arguments and is cached like any other:
     
         >>> t5 = Test()
+        Initializing with () {}
         >>> t5.args
         ()
         >>> t5.kwds
@@ -128,6 +293,7 @@ def cached_class(klass):
     of arguments:
     
         >>> t6 = Test(0)
+        Initializing with (0,) {}
         >>> t6.args
         (0,)
         >>> t6.kwds
@@ -138,6 +304,7 @@ def cached_class(klass):
         >>> t6a is t6
         True
         >>> t7 = Test(a=1)
+        Initializing with () {'a': 1}
         >>> t7.args
         ()
         >>> t7.kwds
@@ -153,11 +320,13 @@ def cached_class(klass):
     the same values:
     
         >>> t8 = Test('a', 'a')
+        Initializing with ('a', 'a') {}
         >>> t8.args
         ('a', 'a')
         >>> t8.kwds
         {}
         >>> t9 = Test(a='a')
+        Initializing with () {'a': 'a'}
         >>> t9.args
         ()
         >>> t9.kwds
@@ -165,6 +334,7 @@ def cached_class(klass):
         >>> t9 is t8
         False
         >>> t10 = Test('a', a='a')
+        Initializing with ('a',) {'a': 'a'}
         >>> t10.args
         ('a',)
         >>> t10.kwds
@@ -178,26 +348,60 @@ def cached_class(klass):
     each time, even for values that compare equal:
     
         >>> t11 = Test([])
+        Initializing with ([],) {}
         >>> t12 = Test([])
+        Initializing with ([],) {}
         >>> t11 is t12
         False
         >>> (t11.args == t12.args) and (t11.kwds == t12.kwds)
         True
         >>> t13 = Test(a=[])
+        Initializing with () {'a': []}
         >>> t14 = Test(a=[])
+        Initializing with () {'a': []}
         >>> t13 is t14
         False
         >>> (t13.args == t14.args) and (t13.kwds == t14.kwds)
         True
         >>> t15 = Test([], a=[])
+        Initializing with ([],) {'a': []}
         >>> t16 = Test([], a=[])
+        Initializing with ([],) {'a': []}
         >>> t15 is t16
         False
         >>> (t15.args == t16.args) and (t15.kwds == t16.kwds)
         True
+    
+    Note that if a cached class is subclassed, the subclass
+    should *not* override the ``__init__`` method (or, of
+    course, the ``__new__`` method). An overridden ``__init__``
+    will get called every time the class is called, even
+    though the same instance will be returned each time:
+    
+        >>> @cached_class
+        ... class TestA(object):
+        ...     def __init__(self, *args, **kwds):
+        ...         print "Initializing with", args, kwds
+        ...
+        >>> a = TestA()
+        Initializing with () {}
+        >>> a1 = TestA()
+        >>> a1 is a
+        True
+        >>> class TestB(Test):
+        ...     def __init__(self, *args, **kwds):
+        ...         print "Initializing again with", args, kwds
+        ...
+        >>> b = TestB()
+        Initializing with () {}
+        Initializing again with () {}
+        >>> b1 = TestB()
+        Initializing again with () {}
+        >>> b1 is b
+        True
     """
     
-    @wraps(klass, assigned=('__name__', '__module__'), updated=())
+    @wraps_class(klass)
     class _decorated(klass):
         # The wraps decorator can't do this because __doc__
         # isn't writable once the class is created
@@ -217,8 +421,9 @@ def cached_class(klass):
             # Python 3 the warning will become an error)
             inst = klass(*args, **kwds)
             # This makes isinstance and issubclass work
-            # properly
-            inst.__class__ = _decorated
+            # properly; we say cls instead of _decorated
+            # so that the decorated class is subclassable
+            inst.__class__ = cls
             return inst
         
         def __init__(self, *args, **kwds):
@@ -275,13 +480,6 @@ def test_required(test_func, fail_func):
     return decorator
 
 
-def merge_dict(target, source):
-    """Merges source into target."""
-    merges = dict((key, value) for key, value in source.iteritems()
-        if key not in target)
-    target.update(merges)
-
-
 def decorator_prep_result(prep_func=None, result_func=None,
         *d_args, **d_kwds):
     """General pattern for decorators with preparation and result functions.
@@ -440,14 +638,13 @@ def decorator_prep_result(prep_func=None, result_func=None,
     return decorator
 
 
-def decorator_with_f(f, decorator):
+def decorator_with_args(decorator):
     """Allows decorator to be used either with or without parameters.
     
     For example, let's modify the above web app login use case::
     
     - We set up a login_required decorator we can use with no arguments,
-      as above (note that we have to give defaults for all arguments, and
-      in particular the first argument ``f`` must default to ``None``)::
+      as above::
     
         >>> users = ['alice', 'bob']
         >>> def test_user(username):
@@ -456,8 +653,9 @@ def decorator_with_f(f, decorator):
         >>> def failed_user(username):
         ...     return username + " is not a known user."
         ... 
-        >>> def login_required(f=None, test=test_user, failed=failed_user):
-        ...     return decorator_with_f(f, test_required(test, failed))
+        >>> @decorator_with_args
+        ... def login_required(f, test=test_user, failed=failed_user):
+        ...     return test_required(test, failed)(f)
         ... 
         >>> @login_required
         ... def user_login(username):
@@ -494,11 +692,26 @@ def decorator_with_f(f, decorator):
         >>> admin_login('charlie')
         'charlie is not a known user.'
         >>> 
+    
+    Note that we only allow keyword arguments for the decorator that
+    this one is applied to; this is to avoid collision between other
+    positional arguments and the ``func`` argument, which will be ``None``
+    if the decorator is being applied with keywords, as in the second
+    example above. Note also that the underlying decorator must ``not``
+    assign a default value to the ``func`` argument (which need not be
+    named ``func``, but it must be the first and only positional argument);
+    whenever the underlying decorator is called, it should be with an
+    argument present there, and assigning a default would suppress a
+    valuable diagnostic if that argument is ever missing.
     """
     
-    if f is None:
-        return decorator
-    return decorator(f)
+    @wraps(decorator)
+    def decofunc(func=None, **kwds):
+        decorated = lambda f: decorator(f, **kwds)
+        if func is None:
+            return decorated
+        return decorated(func)
+    return decofunc
 
 
 def delay(decorator):
@@ -584,15 +797,31 @@ def delay(decorator):
         >>> b.test4()
         Test from TestB
     
+    Note that this meta-decorator automatically includes the
+    functionality of ``decorator_with_args``, above, so you
+    do not need to use that meta-decorator if you use this one.
+    (The extra functionality comes automatically because
+    ``DelayedDecorator`` has to store the full decorator it's
+    delaying, including any keyword arguments; but since those
+    arguments aren't available when the meta-decorator is
+    invoked, the meta-decorator has to add the same extra level
+    of indirection that ``decorator_with_args`` does.)
+    
     See the ``DelayedDecorator`` module for more information.
     """
     
     from plib.stdlib import DelayedDecorator
-    return partial(DelayedDecorator, decorator)
+    @wraps(decorator)
+    def decofunc(func=None, **kwds):
+        decorated = partial(DelayedDecorator, lambda f: decorator(f, **kwds))
+        if func is None:
+            return decorated
+        return decorated(func)
+    return decofunc
 
 
 @delay
-def memoize_generator(gen):
+def memoize_generator(gen, cachelimit=None):
     """Memoizes a generator so each term is only computed once.
     
     Typical usage and comparison with non-memoized generators:
@@ -761,15 +990,86 @@ def memoize_generator(gen):
         Yielding 3
         [0, 3]
     
+    Finally, we can use the ``cachelimit`` keyword argument to
+    limit the size of the cache; elements after the cache limit
+    is reached are not memoized:
+    
+        >>> @memoize_generator(cachelimit=10)
+        ... def numgen():
+        ...     for i in xrange(20):
+        ...         print "Yielding", i
+        ...         yield i
+        ...
+        >>> for i in numgen():
+        ...     print i
+        ...
+        Yielding 0
+        0
+        Yielding 1
+        1
+        Yielding 2
+        2
+        Yielding 3
+        3
+        Yielding 4
+        4
+        Yielding 5
+        5
+        Yielding 6
+        6
+        Yielding 7
+        7
+        Yielding 8
+        8
+        Yielding 9
+        9
+        Yielding 10
+        10
+        Yielding 11
+        11
+        Yielding 12
+        12
+        Yielding 13
+        13
+        Yielding 14
+        14
+        Yielding 15
+        15
+        Yielding 16
+        16
+        Yielding 17
+        17
+        Yielding 18
+        18
+        Yielding 19
+        19
+    
+    Once the elements beyond the cache limit are yielded once, they
+    are not available again:
+    
+        >>> for i in numgen():
+        ...     print i
+        ...
+        0
+        1
+        2
+        3
+        4
+        5
+        6
+        7
+        8
+        9
+    
     See the ``MemoizedGenerator`` module for more information.
     """
     
     from plib.stdlib import MemoizedGenerator
-    return MemoizedGenerator(gen)
+    return MemoizedGenerator(gen, cachelimit=cachelimit)
 
 
 @delay
-def indexable_generator(gen):
+def indexable_generator(gen, cachelimit=None):
     """Make a generator indexable like a sequence.
     
     Typical usage:
@@ -1047,7 +1347,7 @@ def indexable_generator(gen):
     """
     
     from plib.stdlib import IndexedGenerator
-    return IndexedGenerator(gen)
+    return IndexedGenerator(gen, cachelimit=cachelimit)
 
 
 if __name__ == '__main__':
