@@ -35,7 +35,7 @@ function first_n -- generates first n items from iterable
     [0, 1]
     >>> list(first_n(2, xrange(3)))
     [0, 1]
-    >>> list(first_n(1, []))
+    >>> list(first_n(1, xrange(0)))
     []
 
 function pairwise -- generates items from iterable in pairs
@@ -82,31 +82,31 @@ into a list of 1-tuples; also note that n = 0 raises ValueError
 
 function partitions -- generates all partitions of a sequence or set
 
-    >>> list(partitions(range(3)))
+    >>> list(partitions(list(xrange(3))))
     [([0], [1, 2]), ([1], [0, 2]), ([2], [0, 1])]
 
 function subsequences -- generates all subsequences of a sequence,
 from shortest to longest
 
-    >>> list(subsequences(range(2)))
+    >>> list(subsequences(list(xrange(2))))
     [[0], [1], [0, 1]]
-    >>> list(subsequences(range(3)))
+    >>> list(subsequences(list(xrange(3))))
     [[0], [1], [2], [0, 1], [1, 2], [0, 1, 2]]
-    >>> list(subsequences(range(1)))
+    >>> list(subsequences(list(xrange(1))))
     [[0]]
-    >>> list(subsequences(range(0)))
+    >>> list(subsequences(list(xrange(0))))
     []
 
 function inverse_subsequences -- generates all subsequences of a sequence,
 from longest to shortest
 
-    >>> list(inverse_subsequences(range(3)))
+    >>> list(inverse_subsequences(list(xrange(3))))
     [[0, 1, 2], [0, 1], [1, 2], [0], [1], [2]]
-    >>> list(inverse_subsequences(range(2)))
+    >>> list(inverse_subsequences(list(xrange(2))))
     [[0, 1], [0], [1]]
-    >>> list(inverse_subsequences(range(1)))
+    >>> list(inverse_subsequences(list(xrange(1))))
     [[0]]
-    >>> list(inverse_subsequences(range(0)))
+    >>> list(inverse_subsequences(list(xrange(0))))
     []
 
 function cyclic_permutations -- generates cyclic permutations of iterable
@@ -174,51 +174,63 @@ function partition -- splits a sequence into multiple sequences at
 given index (index must be > 0, otherwise ValueError is raised); if
 the ``complete`` argument is ``False``, only partition once
 
-    >>> partition(range(4), 2)
+    >>> partition(list(xrange(4)), 2)
     ([0, 1], [2, 3])
-    >>> partition(range(4), 3)
+    >>> partition(list(xrange(4)), 3)
     ([0, 1, 2], [3])
-    >>> partition(range(4), 4)
+    >>> partition(list(xrange(4)), 3, fill=True)
+    ([0, 1, 2], [3, None, None])
+    >>> partition(list(xrange(4)), 4)
     ([0, 1, 2, 3],)
-    >>> partition(range(4), 5)
+    >>> partition(list(xrange(4)), 4, fill=True)
     ([0, 1, 2, 3],)
-    >>> partition(range(6), 2)
+    >>> partition(list(xrange(4)), 5)
+    ([0, 1, 2, 3],)
+    >>> partition(list(xrange(4)), 5, fill=True)
+    ([0, 1, 2, 3, None],)
+    >>> partition(list(xrange(6)), 2)
     ([0, 1], [2, 3], [4, 5])
-    >>> partition(range(6), 3)
+    >>> partition(list(xrange(6)), 3)
     ([0, 1, 2], [3, 4, 5])
-    >>> partition(range(6), 4)
+    >>> partition(list(xrange(6)), 4)
     ([0, 1, 2, 3], [4, 5])
-    >>> partition(range(6), 5)
+    >>> partition(list(xrange(6)), 5)
     ([0, 1, 2, 3, 4], [5])
-    >>> partition(range(6), 6)
+    >>> partition(list(xrange(6)), 6)
     ([0, 1, 2, 3, 4, 5],)
-    >>> partition(range(6), 7)
+    >>> partition(list(xrange(6)), 7)
     ([0, 1, 2, 3, 4, 5],)
-    >>> partition(range(4), 1)
+    >>> partition(list(xrange(4)), 1)
     ([0], [1], [2], [3])
-    >>> partition(range(1), 1)
+    >>> partition(list(xrange(1)), 1)
     ([0],)
-    >>> partition(range(1), 2)
+    >>> partition(list(xrange(1)), 2)
     ([0],)
-    >>> partition(range(0), 1)
+    >>> partition(list(xrange(1)), 2, fill=True)
+    ([0, None],)
+    >>> partition(list(xrange(0)), 1)
     ([],)
-    >>> partition(range(0), 2)
+    >>> partition(list(xrange(0)), 1, fill=True)
     ([],)
-    >>> partition(range(1), 0)
+    >>> partition(list(xrange(0)), 2)
+    ([],)
+    >>> partition(list(xrange(0)), 2, fill=True)
+    ([],)
+    >>> partition(list(xrange(1)), 0)
     Traceback (most recent call last):
     ...
     ValueError: can't partition sequence at index 0
-    >>> partition(range(0), 0)
+    >>> partition(list(xrange(0)), 0)
     Traceback (most recent call last):
     ...
     ValueError: can't partition sequence at index 0
-    >>> partition(range(3), 1, complete=False)
+    >>> partition(list(xrange(3)), 1, complete=False)
     ([0], [1, 2])
-    >>> partition(range(2), 1, complete=False)
+    >>> partition(list(xrange(2)), 1, complete=False)
     ([0], [1])
-    >>> partition(range(5), 2, complete=False)
+    >>> partition(list(xrange(5)), 2, complete=False)
     ([0, 1], [2, 3, 4])
-    >>> partition(range(4), 2, complete=False)
+    >>> partition(list(xrange(4)), 2, complete=False)
     ([0, 1], [2, 3])
 
 function unzip -- inverse of the zip builtin, splits sequence of tuples
@@ -237,6 +249,41 @@ into multiple sequences
      ...
     TypeError: object of type 'int' has no len()
 
+function group_into -- Generate tuples of every n elements of iterable
+
+    >>> list(group_into(2, xrange(10)))
+    [(0, 1), (2, 3), (4, 5), (6, 7), (8, 9)]
+    >>> list(group_into(3, xrange(10)))
+    [(0, 1, 2), (3, 4, 5), (6, 7, 8)]
+    >>> list(group_into(3, xrange(10), include_tail=True))
+    [(0, 1, 2), (3, 4, 5), (6, 7, 8), (9,)]
+    >>> list(group_into(3, xrange(10), include_tail=True, fill_tail=True))
+    [(0, 1, 2), (3, 4, 5), (6, 7, 8), (9, None, None)]
+    >>> list(group_into(1, xrange(2)))
+    [(0,), (1,)]
+    >>> list(group_into(1, xrange(1)))
+    [(0,)]
+    >>> list(group_into(2, xrange(1)))
+    []
+    >>> list(group_into(2, xrange(1), include_tail=True))
+    [(0,)]
+    >>> list(group_into(2, xrange(1), include_tail=True, fill_tail=True, fillvalue=-1))
+    [(0, -1)]
+    >>> list(group_into(2, xrange(1), raise_if_tail=True))
+    Traceback (most recent call last):
+     ...
+    ValueError: extra terms in grouped iterable
+    >>> list(group_into(0, xrange(1)))
+    Traceback (most recent call last):
+     ...
+    ValueError: can't group iterable into 0-element tuples
+    >>> list(group_into(1, xrange(0)))
+    []
+    >>> list(group_into(1, xrange(0), include_tail=True))
+    []
+    >>> list(group_into(1, xrange(0), include_tail=True, fill_tail=True))
+    []
+
 function exrange -- version of xrange builtin that works with longs
 
     >>> list(exrange(2147483647, 2147483649))
@@ -250,53 +297,55 @@ function exrange -- version of xrange builtin that works with longs
 
 function inverse_index -- index of item counting from end of sequence
 
-    >>> inverse_index(range(3), 0)
+    >>> inverse_index(list(xrange(3)), 0)
     2
-    >>> inverse_index(range(3), 1)
+    >>> inverse_index(list(xrange(3)), 1)
     1
-    >>> inverse_index(range(3), 2)
+    >>> inverse_index(list(xrange(3)), 2)
     0
-    >>> inverse_index(range(3), 3)
+    >>> inverse_index(list(xrange(3)), 3)
     Traceback (most recent call last):
      ...
     ValueError: list.index(x): x not in list
 
 function is_subsequence -- tests if one sequence is subsequence of another
 
-    >>> is_subsequence(range(2), range(3))
+    >>> is_subsequence(list(xrange(2)), list(xrange(3)))
     True
-    >>> is_subsequence(range(2), range(1, 3))
+    >>> is_subsequence(list(xrange(3)), list(xrange(1, 3)))
     False
-    >>> is_subsequence(range(1, 2), range(1, 3))
+    >>> is_subsequence(list(xrange(1, 2)), list(xrange(1, 3)))
     True
-    >>> is_subsequence(range(1), range(2))
+    >>> is_subsequence(list(xrange(1)), list(xrange(2)))
     True
-    >>> is_subsequence(range(1), range(1, 2))
+    >>> is_subsequence(list(xrange(1)), list(xrange(1, 2)))
     False
-    >>> is_subsequence(range(0), range(1))
+    >>> is_subsequence(list(xrange(0)), list(xrange(1)))
     True
-    >>> is_subsequence(range(0), range(0))
+    >>> is_subsequence(list(xrange(0)), list(xrange(0)))
     True
-    >>> is_subsequence(range(3), range(2))
+    >>> is_subsequence(list(xrange(3)), list(xrange(2)))
     False
-    >>> is_subsequence(range(1), range(0))
+    >>> is_subsequence(list(xrange(1)), list(xrange(0)))
     False
 
 function count_matches -- returns mapping of unique elements in sequence
 to number of times they appear
 
-    >>> count_matches(range(1))
-    {0: 1}
-    >>> count_matches(range(2))
-    {0: 1, 1: 1}
-    >>> count_matches(range(2) * 2)
-    {0: 2, 1: 2}
-    >>> count_matches(range(2) * 2 + [2])
-    {0: 2, 1: 2, 2: 1}
-    >>> count_matches(range(0))
-    {}
-    >>> count_matches('abracadabra')
-    {'a': 5, 'c': 1, 'r': 2, 'b': 2, 'd': 1}
+    >>> sorted(count_matches(list(xrange(1))).iteritems())
+    [(0, 1)]
+    >>> sorted(count_matches(list(xrange(2))).iteritems())
+    [(0, 1), (1, 1)]
+    >>> sorted(count_matches(list(xrange(2)) * 2).iteritems())
+    [(0, 2), (1, 2)]
+    >>> sorted(count_matches(list(xrange(2)) * 2 + [1]).iteritems())
+    [(0, 2), (1, 3)]
+    >>> sorted(count_matches(list(xrange(2)) * 2 + [2]).iteritems())
+    [(0, 2), (1, 2), (2, 1)]
+    >>> sorted(count_matches(list(xrange(0))).iteritems())
+    []
+    >>> sorted(count_matches('abracadabra').iteritems())
+    [('a', 5), ('b', 2), ('c', 1), ('d', 1), ('r', 2)]
 
 function sorted_unzip -- returns two sequences from mapping, of keys and
 values, in corresponding order, sorted by key or value
@@ -464,39 +513,71 @@ def unique_permutations_with_replacement(iterable, r):
                         yield u
 
 
-def partition(seq, at, complete=True):
-    # Return two or more sequences, split from seq at given index
+def partition(iterable, at, complete=True, fill=False, fillvalue=None):
+    # Return a tuple of one or more sequences, split from iterable at
+    # given index (if there are fewer elements in iterable, then a
+    # 1-element tuple containing a sequence of all elements is returned).
     # If complete is True and seq is longer than 2 * at, partition
-    # multiple times; otherwise only partition once (i.e., into
+    # multiple times, and fill in the final element if fill is True,
+    # similar to group_into; otherwise only partition once (i.e., into
     # two sequences)
     if at <= 0:
         raise ValueError, "can't partition sequence at index 0"
-    results = []
-    while at < len(seq):
-        results.append(seq[:at])
-        seq = seq[at:]
-        if not complete:
-            break
-    results.append(seq)
-    return tuple(results)
+    if complete:
+        results = tuple(list(g) for g in group_into(at, iterable,
+            include_tail=True, fill_tail=fill, fillvalue=fillvalue))
+        if results:
+            return results
+        return ([],)
+    it = iter(iterable)
+    part = list(islice(it, at))
+    rest = list(it)
+    if not rest:
+        return (part,)
+    return (part, rest)
 
 
-def unzip(seq):
-    # Unzip seq into multiple sequences; assumes seq contains tuples
-    # that are all of the same length. Empty sequences are returned
-    # unchanged.
-    if not seq:
-        return seq
-    num = len(seq[0])
+def unzip(iterable):
+    # Unzip iterable into multiple sequences; assumes iterable
+    # contains tuples that are all of the same length. Empty
+    # sequences are returned unchanged.
+    if not iterable:
+        return iterable
+    num = None
     results = []
-    for _ in xrange(num):
-        results.append([])
-    for items in seq:
+    for items in iterable:
+        if num is None:
+            num = len(items)
+            for _ in xrange(num):
+                results.append([])
         for i in xrange(num):
             results[i].append(items[i])
     return tuple(results)
 
 
+def group_into(n, iterable, include_tail=False, fill_tail=False,
+               fillvalue=None, raise_if_tail=False):
+    # Return tuples of every n elements of iterable
+    # If include_tail is True, yield a last tuple of less than n
+    # elements from iterable if present, filled out with fillvalue
+    # if fill_tail is True; otherwise, if raise_if_tail is True,
+    # raise ValueError if extra elements are present
+    if n < 1:
+        raise ValueError, "can't group iterable into 0-element tuples"
+    args = [iter(iterable)] * n
+    sentinel = object()
+    for group in izip_longest(*args, fillvalue=sentinel):
+        if group[-1] is not sentinel:
+            yield group
+        elif include_tail:
+            tail = tuple(takewhile(lambda g: g is not sentinel, group))
+            if fill_tail:
+                tail += (fillvalue,) * (n - len(tail))
+            yield tail
+        elif raise_if_tail:
+            raise ValueError, "extra terms in grouped iterable"
+
+
 def exrange(start, stop, step=1):
     # xrange that works with Python long ints
     if step == 0:
@@ -544,10 +625,14 @@ def sorted_unzip(mapping, by_value=False, reverse=False):
     # sorted in corresponding order, by values if by_value is
     # True, by keys otherwise; reverse parameter governs the sort
     if by_value:
-        unsorted_result = list((v, k) for k, v in mapping.iteritems())
+        # Do this to avoid a generator expression, for legacy
+        # Python compatibility
+        def unsorted_result():
+            for k, v in mapping.iteritems():
+                yield v, k
     else:
-        unsorted_result = list(mapping.iteritems())
-    result = sorted(unsorted_result, reverse=reverse)
+        unsorted_result = mapping.iteritems
+    result = sorted(unsorted_result(), reverse=reverse)
     r1, r2 = unzip(result)
     if by_value:
         return r2, r1
