@@ -25,7 +25,8 @@ import unittest
 
 modules_with_doctests = [
     'plib.stdlib.decotools',
-    'plib.stdlib.iters'
+    'plib.stdlib.iters',
+    'plib.stdlib._typed_namedtuple'
 ]
 
 try:
