@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 """
 TEST_GUI_SPECS.PY -- test script for module SPECS, sub-package GUI of package PLIB
-Copyright (C) 2008-2011 by Peter A. Donis
+Copyright (C) 2008-2012 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -43,7 +43,8 @@ if specs is not None:
         'items': dummy_list,
         'tabs': dummy_list,
         'contents': dummy_list,
-        'target': dummy_str }
+        'target': dummy_str,
+        'width': dummy_int }
     
     
     def spec(fname, include_defaults=False):
@@ -75,5 +76,6 @@ if specs is not None:
                 self.failUnless(spec_format_ok(spec(specname)))
                 self.failUnless(spec_format_ok(spec(specname, include_defaults=True)))
 
+
 if __name__ == '__main__':
     unittest.main()
