@@ -1,8 +1,7 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """
 TEST_STDLIB.PY -- test script for sub-package STDLIB of package PLIB
-Copyright (C) 2008-2011 by Peter A. Donis
+Copyright (C) 2008-2012 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -12,7 +11,8 @@ This script contains unit tests for the PLIB.STDLIB sub-package.
 
 import unittest
 
-from plib.stdlib import cached_property, fix_newlines, split_string
+from plib.stdlib import fix_newlines, split_string
+from plib.stdlib.decotools import cached_property
 
 
 class CachedPropertyTest(object):
