@@ -80,7 +80,14 @@ if hasattr(signal, 'SIGCHLD'):
             # any possible errno value, but lower than the range PLIB uses
             # for forking errors, so we can tell which error we got
             if not self.child_was_open:
+                # The child shut down early
                 os._exit(198)
+            if self.active_children:
+                # The shutdown process didn't clear active_children
+                os._exit(197)
+            # If we get to here, there was a child and it's no longer in
+            # active_children, so we should be home free, but we're
+            # paranoid so we check further
             try:
                 pid, status = os.waitpid(-1, os.WNOHANG)  # should raise OSError
             except OSError, e:
